! ----------------------------------------------------------------------
! *   Gaussian elimination method for a asymmetric dense matrix        *
! ----------------------------------------------------------------------
      subroutine gausslu(a, ip, n)

      implicit none
      real(8), parameter:: eps = 1.0d-37
      real(8):: a(n,n)
      real(8):: al,piv
      integer:: n
      integer:: ip(n)
      integer:: i, j, k, l
      integer:: lv, isingul
!
!    initialize indices for pivoting
!
      do k = 1,n
        ip(k) = k
      end do

! =====================   k-loop   ===================================
      do k = 1, n-1

!  ****    partial pivot selection    ****

        l = k
        al = abs(a(ip(l),k))

        do i = k+1,n
          if ( abs(a(ip(i),k)) .gt. al ) then
            l = i
            al = abs(a(ip(l),k))
          end if
        end do

!  ****    row exchange    ****

        if ( l .ne. k ) then
          lv = ip(k)
          ip(k) = ip(l)
          ip(l) = lv
        end if

        if ( abs(a(ip(k),k)) .lt.eps ) then
          isingul = 9 
          exit
        end if

!  ****    Gaussian elimination    ****

        piv = 1.0d0 / a(ip(k),k)

        do j = k+1, n
          a(ip(k),j) = a(ip(k),j) * piv
        end do
        do i = k+1, n
          do j = k+1, n
            a(ip(i),j) = a(ip(i),j) - a(ip(i),k) * a(ip(k),j)
          end do
      end do

      end do
! =====================   k-loop end  ================================

!  ****    singular matrix    ****

      if ( isingul.eq.9 ) then
        write (6,*) '(decomp) matrix singular (at ',k,'-th pivot)'
        n = -k
      end if 

      return
      end subroutine gausslu

